/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : standby11.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements the function of Standby processing.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include "standbycommon.h"
#include "standbyinit.h"
#include "standby11.h"

#if defined STANDBY_USED
/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/

/*******************************************************************************
* Function Name: STANDBY11_exec
* Description  : Setting, execution and return of a standby mode.
* Arguments    : none
* Return Value : none
********************************************************************************/
void STANDBY11_exec( UCHAR StandbyMode, USHORT Time, UCHAR In )
{
	if (In == 0)
	{
		return;
	}
	
	/* State save */
	STATE_SAVE();
	
	/* setting for standby mode. */
	OSMC = 0x10;            /* supply fIL               */

	TRTCR0 |= 0x10;         /* supply clock to 8bit IT  */
	TSTART00 = 0;           /* stop 8bit IT counting    */
	ITMK00 = 1;             /* disable interrupt        */

	TRTCR0 |= 0x80;         /* set 16bit operation mode */
	TRTMD0 = 0x03;          /* set count clock fIL/8    */
	TRTCMP0 = ((Time * 150) >> 3 ); /* set counter      */
	
	/* switch the flash operation mode. */
	TRANS_FLASHMODE();
	
	/* Interrupt enable */
	ITIF00 = 0;
	ITMK00 = 0;
	/* 8 bit interval timer start   */
	TSTART00 = 1;
	
	/* Standby mode  Execution. */
	DI();                   /* vector interrupt disable */
	if ( StandbyMode == STANDBY_MODE_STOP ) {
		STOP();
	}
	else
	{
		HALT();
	}
	
	/* 8 bit interval timer stop    */
	TSTART00 = 0;
	/* Interrupt disabled */
	ITIF00 = 0;             /* clear interrupt flag     */
	ITMK00 = 1;             /* mask interrupt           */
	EI();                   /* vector interrupt enable  */
	
	/* switch the flash operation mode. */
	RETURN_FLASHMODE();
	
	/* State return */
	STATE_RETURN();
	
	/* Run mode initializing. */
	STANDBY_INIT0();
	
}
#endif /* end of defined STANDBY_USED */
